@@ -99,9 +99,255 @@ noise = re.compile(""" ّ    | # Tashdīd / Shadda
                        ࣱ    | # Quranic Open Ḍammatān
                        ࣲ    | # Quranic Open Kasratān
                        ٰ    | # Dagger Alif
-                       ـ     # Taṭwīl / Kashīda
+                       ـ   | # Taṭwīl / Kashīda
+                       ٖ    | # ARABIC SUBSCRIPT ALEF
+                       ٗ    | # ARABIC INVERTED DAMMA
+                       ۡ    | # ARABIC SMALL HIGH DOTLESS HEAD OF KHAH = Qur'anic sukūn
+                       ۤ      # ARABIC SMALL HIGH MADDA
                    """, re.VERBOSE)
 
+# Whitelist of characters that are allowed in OpenITI texts:
+
+allowed_chars = """\
+ء	ARABIC LETTER HAMZA
+آ	ARABIC LETTER ALEF WITH MADDA ABOVE
+أ	ARABIC LETTER ALEF WITH HAMZA ABOVE
+ؤ	ARABIC LETTER WAW WITH HAMZA ABOVE
+إ	ARABIC LETTER ALEF WITH HAMZA BELOW
+ئ	ARABIC LETTER YEH WITH HAMZA ABOVE
+ا	ARABIC LETTER ALEF
+ب	ARABIC LETTER BEH
+ة	ARABIC LETTER TEH MARBUTA
+ت	ARABIC LETTER TEH
+ث	ARABIC LETTER THEH
+ج	ARABIC LETTER JEEM
+ح	ARABIC LETTER HAH
+خ	ARABIC LETTER KHAH
+د	ARABIC LETTER DAL
+ذ	ARABIC LETTER THAL
+ر	ARABIC LETTER REH
+ز	ARABIC LETTER ZAIN
+س	ARABIC LETTER SEEN
+ش	ARABIC LETTER SHEEN
+ص	ARABIC LETTER SAD
+ض	ARABIC LETTER DAD
+ط	ARABIC LETTER TAH
+ظ	ARABIC LETTER ZAH
+ع	ARABIC LETTER AIN
+غ	ARABIC LETTER GHAIN
+ف	ARABIC LETTER FEH
+ق	ARABIC LETTER QAF
+ك	ARABIC LETTER KAF
+ل	ARABIC LETTER LAM
+م	ARABIC LETTER MEEM
+ن	ARABIC LETTER NOON
+ه	ARABIC LETTER HEH
+و	ARABIC LETTER WAW
+ى	ARABIC LETTER ALEF MAKSURA
+ي	ARABIC LETTER YEH
+٠	ARABIC-INDIC DIGIT ZERO
+١	ARABIC-INDIC DIGIT ONE
+٢	ARABIC-INDIC DIGIT TWO
+٣	ARABIC-INDIC DIGIT THREE
+٤	ARABIC-INDIC DIGIT FOUR
+٥	ARABIC-INDIC DIGIT FIVE
+٦	ARABIC-INDIC DIGIT SIX
+٧	ARABIC-INDIC DIGIT SEVEN
+٨	ARABIC-INDIC DIGIT EIGHT
+٩	ARABIC-INDIC DIGIT NINE
+ٹ	ARABIC LETTER TTEH
+پ	ARABIC LETTER PEH
+چ	ARABIC LETTER TCHEH
+ڈ	ARABIC LETTER DDAL
+ڑ	ARABIC LETTER RREH
+ژ	ARABIC LETTER JEH
+ک	ARABIC LETTER KEHEH
+ݣ	ARABIC LETTER KEHEH WITH THREE DOTS ABOVE
+ڭ	ARABIC LETTER NG
+گ	ARABIC LETTER GAF
+ۀ	ARABIC LETTER HEH WITH YEH ABOVE
+ہ	ARABIC LETTER HEH GOAL
+ۂ	ARABIC LETTER HEH GOAL WITH HAMZA ABOVE
+ی	ARABIC LETTER FARSI YEH
+ے	ARABIC LETTER YEH BARREE
+ۓ	ARABIC LETTER YEH BARREE WITH HAMZA ABOVE
+ھ	ARABIC LETTER HEH DOACHASHMEE
+ە	ARABIC LETTER AE
+ں	ARABIC LETTER NOON GHUNNA
+ٮ   ARABIC LETTER DOTLESS BEH
+ٯ   ARABIC LETTER DOTLESS QAF
+ڡ   ARABIC LETTER DOTLESS FEH
+ڣ   ARABIC LETTER FEH WITH DOT BELOW
+‌	ZERO WIDTH NON-JOINER
+ٔ	ARABIC HAMZA ABOVE (needed for the Farsi izafeh)
+#	NUMBER SIGN
+%	PERCENT SIGN
+(	LEFT PARENTHESIS
+)	RIGHT PARENTHESIS
+.	FULL STOP
+/	SOLIDUS
+0	DIGIT ZERO
+1	DIGIT ONE
+2	DIGIT TWO
+3	DIGIT THREE
+4	DIGIT FOUR
+5	DIGIT FIVE
+7	DIGIT SEVEN
+8	DIGIT EIGHT
+9	DIGIT NINE
+:	COLON
+|	VERTICAL LINE
+~	TILDE
+؟	ARABIC QUESTION MARK
+،	ARABIC COMMA
+!	EXCLAMATION MARK
+$	DOLLAR SIGN
+*	ASTERISK
+-	HYPHEN-MINUS
+_	LOW LINE (i.e., underscore)
+«	LEFT-POINTING DOUBLE ANGLE QUOTATION MARK
+»	RIGHT-POINTING DOUBLE ANGLE QUOTATION MARK
+؛	ARABIC SEMICOLON
+!	EXCLAMATION MARK
+"	QUOTATION MARK
+,	COMMA
+=	EQUALS SIGN
+?	QUESTION MARK
+“	LEFT DOUBLE QUOTATION MARK
+”	RIGHT DOUBLE QUOTATION MARK
+’	RIGHT SINGLE QUOTATION MARK
+&   AMPERSAND
+§   SECTION SIGN
+¶	PILCROW SIGN
+¬	NOT SIGN
+•	BULLET
+<	LESS-THAN SIGN
+>	GREATER-THAN SIGN
+{	LEFT CURLY BRACKET
+}	RIGHT CURLY BRACKET
++	PLUS SIGN
+;	SEMICOLON
+@	COMMERCIAL AT
+ְ	HEBREW POINT SHEVA
+ֳ	HEBREW POINT HATAF QAMATS
+ִ	HEBREW POINT HIRIQ
+ֵ	HEBREW POINT TSERE
+ֶ	HEBREW POINT SEGOL
+ַ	HEBREW POINT PATAH
+ָ	HEBREW POINT QAMATS
+ֹ	HEBREW POINT HOLAM
+ּ	HEBREW POINT DAGESH OR MAPIQ
+א	HEBREW LETTER ALEF
+ב	HEBREW LETTER BET
+ג	HEBREW LETTER GIMEL
+ד	HEBREW LETTER DALET
+ה	HEBREW LETTER HE
+ו	HEBREW LETTER VAV
+ז	HEBREW LETTER ZAYIN
+ח	HEBREW LETTER HET
+ט	HEBREW LETTER TET
+י	HEBREW LETTER YOD
+ך	HEBREW LETTER FINAL KAF
+כ	HEBREW LETTER KAF
+ל	HEBREW LETTER LAMED
+ם	HEBREW LETTER FINAL MEM
+מ	HEBREW LETTER MEM
+ן	HEBREW LETTER FINAL NUN
+נ	HEBREW LETTER NUN
+ס	HEBREW LETTER SAMEKH
+ע	HEBREW LETTER AYIN
+ף	HEBREW LETTER FINAL PE
+פ	HEBREW LETTER PE
+ץ	HEBREW LETTER FINAL TSADI
+צ	HEBREW LETTER TSADI
+ק	HEBREW LETTER QOF
+ר	HEBREW LETTER RESH
+ש	HEBREW LETTER SHIN
+ת	HEBREW LETTER TAV
+І	CYRILLIC CAPITAL LETTER BYELORUSSIAN-UKRAINIAN I
+А	CYRILLIC CAPITAL LETTER A
+Б	CYRILLIC CAPITAL LETTER BE
+В	CYRILLIC CAPITAL LETTER VE
+Г	CYRILLIC CAPITAL LETTER GHE
+Д	CYRILLIC CAPITAL LETTER DE
+Е	CYRILLIC CAPITAL LETTER IE
+Ж	CYRILLIC CAPITAL LETTER ZHE
+З	CYRILLIC CAPITAL LETTER ZE
+И	CYRILLIC CAPITAL LETTER I
+Й	CYRILLIC CAPITAL LETTER SHORT I
+К	CYRILLIC CAPITAL LETTER KA
+Л	CYRILLIC CAPITAL LETTER EL
+М	CYRILLIC CAPITAL LETTER EM
+Н	CYRILLIC CAPITAL LETTER EN
+О	CYRILLIC CAPITAL LETTER O
+П	CYRILLIC CAPITAL LETTER PE
+Р	CYRILLIC CAPITAL LETTER ER
+С	CYRILLIC CAPITAL LETTER ES
+Т	CYRILLIC CAPITAL LETTER TE
+У	CYRILLIC CAPITAL LETTER U
+Ф	CYRILLIC CAPITAL LETTER EF
+Х	CYRILLIC CAPITAL LETTER HA
+Ц	CYRILLIC CAPITAL LETTER TSE
+Ч	CYRILLIC CAPITAL LETTER CHE
+Ш	CYRILLIC CAPITAL LETTER SHA
+Щ	CYRILLIC CAPITAL LETTER SHCHA
+Ъ	CYRILLIC CAPITAL LETTER HARD SIGN
+Ы	CYRILLIC CAPITAL LETTER YERU
+Ь	CYRILLIC CAPITAL LETTER SOFT SIGN
+Э	CYRILLIC CAPITAL LETTER E
+Ю	CYRILLIC CAPITAL LETTER YU
+Я	CYRILLIC CAPITAL LETTER YA
+а	CYRILLIC SMALL LETTER A
+б	CYRILLIC SMALL LETTER BE
+в	CYRILLIC SMALL LETTER VE
+г	CYRILLIC SMALL LETTER GHE
+д	CYRILLIC SMALL LETTER DE
+е	CYRILLIC SMALL LETTER IE
+ж	CYRILLIC SMALL LETTER ZHE
+з	CYRILLIC SMALL LETTER ZE
+и	CYRILLIC SMALL LETTER I
+й	CYRILLIC SMALL LETTER SHORT I
+к	CYRILLIC SMALL LETTER KA
+л	CYRILLIC SMALL LETTER EL
+м	CYRILLIC SMALL LETTER EM
+н	CYRILLIC SMALL LETTER EN
+о	CYRILLIC SMALL LETTER O
+п	CYRILLIC SMALL LETTER PE
+р	CYRILLIC SMALL LETTER ER
+с	CYRILLIC SMALL LETTER ES
+т	CYRILLIC SMALL LETTER TE
+у	CYRILLIC SMALL LETTER U
+ф	CYRILLIC SMALL LETTER EF
+х	CYRILLIC SMALL LETTER HA
+ц	CYRILLIC SMALL LETTER TSE
+ч	CYRILLIC SMALL LETTER CHE
+ш	CYRILLIC SMALL LETTER SHA
+щ	CYRILLIC SMALL LETTER SHCHA
+ъ	CYRILLIC SMALL LETTER HARD SIGN
+ы	CYRILLIC SMALL LETTER YERU
+ь	CYRILLIC SMALL LETTER SOFT SIGN
+э	CYRILLIC SMALL LETTER E
+ю	CYRILLIC SMALL LETTER YU
+я	CYRILLIC SMALL LETTER YA
+і	CYRILLIC SMALL LETTER BYELORUSSIAN-UKRAINIAN I
+Ѣ	CYRILLIC CAPITAL LETTER YAT
+ѣ	CYRILLIC SMALL LETTER YAT
+Ѳ	CYRILLIC CAPITAL LETTER FITA
+"""
+allowed_chars = [x.split("\t")[0] for x in allowed_chars.splitlines()]
+allowed_chars = [c for c in allowed_chars if c not in ("-", ".")]
+
+# In addition to the above characters, also include Latin script, whitespace and punctuation:
+transcription_chars = "0-9a-zA-ZāĀăĂēĒĕĔṭṬṯṮūŪīĪĭĬİıōŌṣṢšŠḍḌḏḎǧǦġĠğĞḫḪḥḤḳḲẓẒžŽčČçÇñÑãÃáÁàÀäÄéÉèÈêÊëËïÏîÎôÔóÓòÒōÕöÖüÜûÛúÚùÙʿʾ' "
+escaped_chars = r"\"\n\t\[\]\.\-\\"
+
+# build a regex to match all allowed characters (and all those that are not allowed):
+allowed_chars_regex = re.compile(r"[{}{}{}]+".format("".join(allowed_chars), transcription_chars, escaped_chars))
+unwanted_chars_regex = re.compile(r"[^{}{}{}]+".format("".join(allowed_chars), transcription_chars, escaped_chars))
+
+
+
+
 def denoise(text):
     """Remove non-consonantal characters from Arabic text.
 
@@ -439,5 +685,30 @@ def tokenize(text, token_regex=ar_tok):
 
     return tokens, tokenStarts, tokenEnds
 
+
+def decode_unicode_name(s):
+    """Convert unicode names into the characters they refer to.
+
+    Args:
+        s (str): input string
+    
+    Returns:
+        str
+
+    Examples:
+        >>> decode_unicode_name("ARABIC LETTER ALEF_ARABIC LETTER YEH")
+        'اي'
+
+        >>> decode_unicode_name("ARABIC LETTER ALEF_*")
+        'ا*'
+    """
+    new_s = ""
+    for x in s.split("_"):
+        try:
+            new_s += unicodedata.lookup(x)
+        except:
+            new_s += x
+    return new_s
+
 if __name__ == "__main__":
     doctest.testmod()
