@@ -222,8 +222,10 @@ Examples:
 """
 
 import copy
+import json
 import os
 import re
+import requests
 import shutil
 
 if __name__ == '__main__':
@@ -1093,7 +1095,7 @@ Did you put a dot between date and author name?"
             elif self.author:
                 return self.build_pth(uri_type="author", base_pth=base_pth)
             elif self.date:
-                return self.build_pth("date", base_pth)
+                return self.build_pth("date", base_pth=base_pth)
 
 
         if uri_type == "date":
@@ -1104,7 +1106,13 @@ Did you put a dot between date and author name?"
                     d = "{:04d}AH".format((int(int(self.date)/25) + 1)*25)
                 else:
                     d = "{:04d}AH".format(int(self.date))
-                #p = os.path.join(base_pth, d)
+                # take into account the different repo name formats for Arabic and other languages:
+                # (Arabic: 0025AH, 0050AH, ...; Persian: PER0025AH, PER0050AH, ...; Urdu: URD0025AH, ...)
+                try:
+                    if self.language != "ara":
+                        d = self.language.upper() + d
+                except:
+                    pass
                 return os.sep.join((base_pth, d))
             else:
                 raise Exception("Error: the date component of the URI was not defined")
@@ -1139,7 +1147,9 @@ Did you put a dot between date and author name?"
 # OpenITI corpus functions dependent on URIs:
 
 
-def change_uri(old, new, old_base_pth=None, new_base_pth=None, execute=False):
+def change_uri(old, new, old_base_pth=None, new_base_pth=None, execute=False,
+               book_relations_fp="https://github.com/OpenITI/kitab-metadata-automation/raw/master/output/OpenITI_Github_clone_book_relations.json",
+               non_25Y_folder=None):
     """Change a uri and put all files in the correct folder.
 
     If a version URI changes:
@@ -1153,6 +1163,8 @@ def change_uri(old, new, old_base_pth=None, new_base_pth=None, execute=False):
         * all annotation text files of all versions of the book should be moved
         * all yml files of versions of that book should be updated and moved
         * the original book folder itself should be (re)moved
+        * (optionally) all references to the book in book yml files of other books
+          should be updated
         
     if an author uri changes:
 
@@ -1189,8 +1201,10 @@ def change_uri(old, new, old_base_pth=None, new_base_pth=None, execute=False):
         None
     """
     print("old_base_pth:", old_base_pth)
+    old = old.strip()
     old_uri = URI(old)
     old_uri.base_pth = old_base_pth
+    new = new.strip()
     new_uri = URI(new)
     new_uri.base_pth = new_base_pth
 
@@ -1206,15 +1220,19 @@ def change_uri(old, new, old_base_pth=None, new_base_pth=None, execute=False):
             if not file.endswith(".md"):
                 if URI(file).build_uri(ext="") == old_uri.build_uri(ext=""):
                     if file.endswith(".yml"):
-                        move_yml(fp, new_uri, "version", execute)
+                        move_yml(fp, new_uri, "version", execute, 
+                                 non_25Y_folder=non_25Y_folder)
                     else:
                         old_file_uri = URI(fp)
                         new_uri.extension = old_file_uri.extension
-                        move_to_new_uri_pth(fp, new_uri, execute)
+                        move_to_new_uri_pth(fp, new_uri, execute,
+                                            non_25Y_folder=non_25Y_folder)
 
         # add readme and text_questionnaire files:
 
         target_folder = new_uri.build_pth("version")
+        if non_25Y_folder:
+            target_folder = re.sub("\d{4}AH", non_25Y_folder, target_folder)
         if execute:
             if "README.md" not in os.listdir(target_folder):
                 add_readme(target_folder)
@@ -1244,9 +1262,11 @@ def change_uri(old, new, old_base_pth=None, new_base_pth=None, execute=False):
                         new_file_uri.title = new_uri.title
                     if file.endswith(".yml"):
                         new_fp = move_yml(fp, new_file_uri,
-                                          old_file_uri.uri_type, execute)
+                                          old_file_uri.uri_type, execute,
+                                          non_25Y_folder=non_25Y_folder)
                     else:
-                        new_fp = move_to_new_uri_pth(fp, new_file_uri, execute)
+                        new_fp = move_to_new_uri_pth(fp, new_file_uri, execute,
+                                                     non_25Y_folder=non_25Y_folder)
 
             # Deal with non-URI filenames last:
 
@@ -1262,30 +1282,68 @@ def change_uri(old, new, old_base_pth=None, new_base_pth=None, execute=False):
                     else:
                         print("  Move", fp, "to", new_fp)
 
+    # change references to the book in book relations sections of book yml files:
+
+    if new_uri.uri_type == "book" and book_relations_fp is not None:
+        print("CHECKING BOOK RELATIONS...")
+        if book_relations_fp.startswith("http"):
+            book_relations_d = requests.get(book_relations_fp).json()
+        else:
+            book_relations_d = json.load(book_relations_fp)
+        if old in book_relations_d:
+            print("Old URI found in book relations!")
+            for rel_d in book_relations_d[old]:
+                print(rel_d)
+                if rel_d["dest"].strip() == old:
+                    rel_uri = URI(rel_d["source"])
+                    rel_uri.base_pth = old_base_pth
+                    rel_yml_fp = rel_uri.build_pth(uri_type="book_yml")
+                    with open(rel_yml_fp, mode="r", encoding="utf-8") as file:
+                        data = file.read()
+                    new_data = re.sub(r"\b{}\b".format(old), new, data)
+                    if new_data != data:
+                        if new_base_pth:
+                            rel_uri.base_pth = new_base_pth
+                        rel_yml_fp = rel_uri.build_pth(uri_type="book_yml")
+                        print("Replace book uri in:", rel_yml_fp)
+                        print(new_data)
+                        if execute:
+                            with open(rel_yml_fp, mode="w", encoding="utf-8") as file:
+                                file.write(new_data)
+            
+
     # Remove folders:
 
     if new_uri.uri_type == "author":
-        if execute:
-            shutil.rmtree(old_folder)
+        if os.path.exists(old_folder):
+            if execute:
+                shutil.rmtree(old_folder)
+            else:
+                for book_dir in os.listdir(old_folder):
+                    pth = os.path.join(old_folder, book_dir)
+                    print("REMOVE BOOK FOLDER", pth)
+                print("REMOVE AUTHOR FOLDER", old_folder)
         else:
-            for book_dir in os.listdir(old_folder):
-                if not book_dir.endswith("yml"):
-                    print("REMOVE BOOK FOLDER", os.path.join(old_folder, book_dir))
-            print("REMOVE AUTHOR FOLDER", old_folder)
+            print("Old folder", old_folder, "does not exist!")
     if new_uri.uri_type == "book":
-        if execute:
-            shutil.rmtree(old_folder)
+        if os.path.exists(old_folder):
+            if execute:
+                shutil.rmtree(old_folder)
+            else:
+                print("REMOVE BOOK FOLDER", old_folder)
         else:
-            print("REMOVE BOOK FOLDER", old_folder)
+            print("Old folder", old_folder, "does not exist!")
 
     if not execute:
         resp = input("To carry out these changes: press OK+Enter; \
 to abort: press Enter. ")
         if resp == "OK":
             print()
-            change_uri(old, new, old_base_pth, new_base_pth, execute=True)
+            change_uri(old, new, old_base_pth, new_base_pth, 
+                       execute=True, non_25Y_folder=non_25Y_folder)
         else:
             print("User aborted carrying out these changes!")
+    print("Done")
 
 def add_readme(target_folder):
     """Add default README.md file to target_folder.
@@ -1307,7 +1365,7 @@ def add_text_questionnaire(target_folder):
               mode="w", encoding="utf-8") as file:
         file.write(text_questionnaire_template)
 
-def add_character_count(tok_count, char_count, tar_uri, execute=False):
+def add_character_count(tok_count, char_count, tar_uri, execute=False, non_25Y_folder=None):
     """Add the character and token counts to the new version yml file
 
     Args:
@@ -1316,12 +1374,17 @@ def add_character_count(tok_count, char_count, tar_uri, execute=False):
         tar_uri (URI object): uri of the target text
         execute (bool): if True, the function will do its work silently.
           If False, it will only print a description of the action.
+        non_25Y_folder (str): name of the parent folder for the new files,
+            to be used instead of the 25 years folder (0025AH, 0050AH, ...).
+            Defaults to None (that is: use the auto-generated 25 years folder)
 
     Returns:
         None
     """
 
     tar_yfp = tar_uri.build_pth("version_yml")
+    if non_25Y_folder:
+        tar_yfp = re.sub("\d{4}AH", non_25Y_folder, tar_yfp)
     if execute:
         with open(tar_yfp, mode="r", encoding="utf-8") as file:
             yml_dic = yml.ymlToDic(file.read().strip())
@@ -1358,7 +1421,8 @@ def new_yml(tar_yfp, yml_type, execute=False):
             created_ymls.append(tar_yfp)
 
 
-def move_yml(yml_fp, new_uri, uri_type, execute=False):
+def move_yml(yml_fp, new_uri, uri_type, execute=False,
+             non_25Y_folder=None):
     """Replace the URI in the yml file
     and save the yml file in its new location.
 
@@ -1376,7 +1440,7 @@ def move_yml(yml_fp, new_uri, uri_type, execute=False):
     """
     new_yml_fp = new_uri.build_pth(uri_type=uri_type+"_yml")
     new_yml_folder = os.path.split(new_yml_fp)[0]
-    make_folder(new_yml_folder, new_uri, execute)
+    make_folder(new_yml_folder, new_uri, execute, non_25Y_folder=non_25Y_folder)
 
     if not execute:
         print("  Change URI inside yml file:")
@@ -1397,7 +1461,7 @@ def move_yml(yml_fp, new_uri, uri_type, execute=False):
     return new_yml_fp
 
 
-def make_folder(new_folder, new_uri, execute=False):
+def make_folder(new_folder, new_uri, execute=False, non_25Y_folder=None):
     """Check if folder exists; if not, make folder (and, if needed, parents)
 
     Args:
@@ -1417,6 +1481,8 @@ Make sure base path is correct.""".format(new_uri.base_pth)
         raise Exception(msg)
     if not os.path.exists(new_folder):
         author_folder = new_uri.build_pth("author")
+        if non_25Y_folder:
+            author_folder = re.sub("\d{4}AH", non_25Y_folder, author_folder)
         if not os.path.exists(author_folder):
             if execute:
                 os.makedirs(author_folder)
@@ -1427,6 +1493,8 @@ Make sure base path is correct.""".format(new_uri.base_pth)
             new_yml(new_uri.build_pth("author_yml"), "author_yml", execute)
         if new_uri.uri_type == "book" or new_uri.uri_type == "version":
             book_folder = new_uri.build_pth("book")
+            if non_25Y_folder:
+                book_folder = re.sub("\d{4}AH", non_25Y_folder, book_folder)
             if not os.path.exists(book_folder):
                 if execute:
                     os.makedirs(book_folder)
@@ -1439,14 +1507,17 @@ Make sure base path is correct.""".format(new_uri.base_pth)
                 new_yml(new_uri.build_pth("version_yml"),
                             "version_yml", execute)
                 target_folder = new_uri.build_pth("version")
+                if non_25Y_folder:
+                    target_folder = re.sub("\d{4}AH", non_25Y_folder, target_folder)
                 if execute:
                     if "README.md" not in os.listdir(target_folder):
                         add_readme(target_folder)
                     if "text_questionnaire.md" not in os.listdir(target_folder):
                         add_text_questionnaire(target_folder)
+             
 
 
-def move_to_new_uri_pth(old_fp, new_uri, execute=False):
+def move_to_new_uri_pth(old_fp, new_uri, execute=False, non_25Y_folder=None):
     """Move file to its new location.
 
     Args:
@@ -1456,13 +1527,19 @@ def move_to_new_uri_pth(old_fp, new_uri, execute=False):
             (the user will still be given the option to execute
             all proposed changes at the end);
             if True, all changes will be executed immediately.
+        non_25Y_folder (str): name of the parent folder for the new files,
+            to be used instead of the 25 years folder (0025AH, 0050AH, ...).
+            Defaults to None (that is: use the auto-generated 25 years folder)
 
     Returns:
         (str): path to the new file
     """
     new_folder = new_uri.build_pth(uri_type=new_uri.uri_type)
     new_fp = new_uri.build_pth(uri_type=new_uri.uri_type+"_file")
-    make_folder(new_folder, new_uri, execute)
+    if non_25Y_folder:
+        new_folder = re.sub("\d{4}AH", non_25Y_folder, new_folder)
+        new_fp = re.sub("\d{4}AH", non_25Y_folder, new_fp)
+    make_folder(new_folder, new_uri, execute, non_25Y_folder=non_25Y_folder)
     if execute:
         shutil.move(old_fp, new_fp)
         print("  Move", old_fp, "\n    to", new_fp)
@@ -1507,7 +1584,8 @@ def check_token_count(version_uri, ymlD, version_fp="", find_latest=True):
             version_uri.extension = ext
             fp = version_uri.build_pth(uri_type="version_file")
             if os.path.exists(fp):
-                break
+                break 
+
     tok_count = ar_cnt_file(fp, mode="token")
     char_count = ar_cnt_file(fp, mode="char")
     len_key = "00#VERS#LENGTH###:"
