@@ -264,7 +264,7 @@ class BokJsonConverter(generic_converter.GenericConverter):
         
         shamid = re.findall("\d+", source_fp)[-1]
 
-        book_data, toc_data, metadata = self.get_data(source_fp, shamid)
+        book_data, toc_data, metadata, shamid = self.get_data(source_fp, shamid)
 
         metadata = self.format_metadata(metadata)
 
@@ -274,6 +274,7 @@ class BokJsonConverter(generic_converter.GenericConverter):
         text = self.post_process(text)
         notes = self.reflow(notes)
         notes = re.sub("(\n+)(?![\n~P])", r"\1# ", notes)
+        notes = re.sub("(?:[\r\n]*# )? *LINE_END *", "\n# ", notes)
         text = self.compose(metadata, text, notes)
         #print(9, text)
 
@@ -405,7 +406,12 @@ class BokJsonConverter(generic_converter.GenericConverter):
         with open(source_fp, mode="r", encoding="utf-8") as file:
             data = json.load(file)
 
-        book_data = data["b"+shamid]
+        try:
+            book_data = data["b"+shamid]
+        except:
+            # sometimes, the ID number on the website is not the id number inside the file:
+            shamid = [k[1:] for k in data if re.match("b\d+", k)][0]
+            book_data = data["b"+shamid]
 
         # load the table of contents:
         toc_data = data["t"+shamid]
@@ -422,7 +428,7 @@ class BokJsonConverter(generic_converter.GenericConverter):
         else:
             book_meta = data["Main"]
 
-        return book_data, toc_data, book_meta
+        return book_data, toc_data, book_meta, shamid
 
     def format_text(self, book, toc, book_id):
         """Extract text from json, add page numbers and headings.
@@ -556,10 +562,10 @@ class BokJsonConverter(generic_converter.GenericConverter):
                 tit = deNoise(el["tit"])
             except:
                 tit = "[NO TITLE]"
-            #print("title", tit)
+            # create a regex for the title that disregards non-alphanumeric characters:
             tit_regex = re.sub("(\w+)", r"W\1", tit)[1:]
             tit_regex = re.sub("\W+", r"", tit_regex)
-            tit_regex = re.sub("W", r"\W*", tit_regex)
+            tit_regex = re.sub("W", r"\\W*", tit_regex)
             tit_regex = ".*{}.*".format(tit_regex)
             #print("regex", tit_regex)
             if re.findall(tit_regex, passage_text):
@@ -578,6 +584,11 @@ class BokJsonConverter(generic_converter.GenericConverter):
         """Convert the metadata in the metadata dict to a string."""
         m = []
         fmt = "#META# {}: {}"
+        try:
+            for k,v in metadata.items():
+                pass
+        except: 
+            metadata = metadata[0]
         for k,v in metadata.items():
             if v:
                 # treat the "betaka" field different from the other fields.
@@ -600,6 +611,7 @@ class BokJsonConverter(generic_converter.GenericConverter):
         for k,v in self.additional_meta.items():
             m.append(fmt.format(k,v))
         metadata = "\n\n".join(m)
+        metadata = re.sub("(?: *LINE_END *)+", " ¶ ", metadata)
         return self.magic_value + metadata + self.header_splitter
 
     def pre_process(self, text):
@@ -620,6 +632,7 @@ class BokJsonConverter(generic_converter.GenericConverter):
         Returns:
             texgt (str): the text after post-processing operations.
         """
+        text = re.sub(" *LINE_END *", "\n# ", text)
         # remove unwanted spaces:
         text = re.sub("  +", " ", text)
         text = re.sub("\n ", "\n", text)
@@ -642,6 +655,8 @@ class BokJsonConverter(generic_converter.GenericConverter):
         text = re.sub("\n{2,}", "\n", text)
         text = re.sub("[\r\n]+# *(?:[\r\n]+|\Z)", "\n", text)
 
+        text = re.sub(" ### ", "\n### ", text)
+
         return text
 
 
